@@ -113,15 +113,26 @@ ExecuteCmd::ExecuteCmd(const string &cmd, int screen_num):m_cmd(cmd), m_screen_n
 }
 
 void ExecuteCmd::execute() {
-#ifndef __EMX__
     run();
-#else //   __EMX__
-    spawnlp(P_NOWAIT, "cmd.exe", "cmd.exe", "/c", m_cmd.c_str(), static_cast<void*>(NULL));
-#endif // !__EMX__
-
 }
 
 int ExecuteCmd::run() {
+#if defined(__EMX__) || defined(_WIN32)
+#ifndef PATH_MAX
+#define PATH_MAX 1024
+#endif
+    char comspec[PATH_MAX] = {0};
+    char * env_var = getenv("COMSPEC");
+    if (env_var != NULL) {
+        strncpy(comspec, env_var, PATH_MAX - 1);
+        comspec[PATH_MAX - 1] = '\0';
+    } else {
+        strncpy(comspec, "cmd.exe", 7);
+        comspec[7] = '\0';
+    }
+
+    return spawnlp(P_NOWAIT, comspec, comspec, "/c", m_cmd.c_str(), static_cast<void*>(NULL));
+#else
     pid_t pid = fork();
     if (pid)
         return pid;
@@ -161,6 +172,7 @@ int ExecuteCmd::run() {
     exit(EXIT_SUCCESS);
 
     return pid; // compiler happy -> we are happy ;)
+#endif
 }
 
 FbTk::Command<void> *ExportCmd::parse(const string &command, const string &args,
@@ -433,13 +445,12 @@ REGISTER_UNTRUSTED_COMMAND_WITH_ARGS(bindkey, FbCommands::BindKeyCmd, void);
 BindKeyCmd::BindKeyCmd(const string &keybind):m_keybind(keybind) { }
 
 void BindKeyCmd::execute() {
-    if (Fluxbox::instance()->keys() != 0) {
-        if (Fluxbox::instance()->keys()->addBinding(m_keybind)) {
-            ofstream ofile(Fluxbox::instance()->keys()->filename().c_str(), ios::app);
-            if (!ofile)
-                return;
-            ofile<<m_keybind<<endl;
-        }
+    Keys* keys = Fluxbox::instance()->keys();
+    if (keys && keys->addBinding(m_keybind)) {
+        ofstream ofile(keys->filename().c_str(), ios::app);
+        if (!ofile)
+            return;
+        ofile<<m_keybind<<endl;
     }
 }
 
@@ -500,7 +511,7 @@ void DeiconifyCmd::execute() {
 
     case ALL:
     case ALLWORKSPACE:
-        for(; it != itend; it++) {
+        for(; it != itend; ++it) {
             old_workspace_num= (*it)->workspaceNumber();
             if (m_mode == ALL || old_workspace_num == workspace_num ||
                 (*it)->isStuck()) {
@@ -514,7 +525,7 @@ void DeiconifyCmd::execute() {
     case LAST:
     case LASTWORKSPACE:
     default:
-        for (; it != itend; it++) {
+        for (; it != itend; ++it) {
             old_workspace_num= (*it)->workspaceNumber();
             if(m_mode == LAST || old_workspace_num == workspace_num ||
                (*it)->isStuck()) {
@@ -530,4 +541,73 @@ void DeiconifyCmd::execute() {
     };
 }
 
+
+REGISTER_COMMAND_WITH_ARGS(clientpatterntest, FbCommands::ClientPatternTestCmd, void);
+
+void ClientPatternTestCmd::execute() {
+
+    std::vector< const FluxboxWindow* > matches;
+    std::string                         result;
+    std::string                         pat;
+    int                                 opts;
+    ClientPattern*                      cp;
+    Display*                            dpy;
+    Atom                                atom_utf8;
+    Atom                                atom_fbcmd_result;
+    Fluxbox::ScreenList::const_iterator screen;
+    const Fluxbox::ScreenList           screens(Fluxbox::instance()->screenList());
+
+    dpy = Fluxbox::instance()->display();
+    atom_utf8 = XInternAtom(dpy, "UTF8_STRING", False);
+    atom_fbcmd_result = XInternAtom(dpy, "_FLUXBOX_ACTION_RESULT", False);
+
+    FocusableList::parseArgs(m_args, opts, pat);
+    cp = new ClientPattern(pat.c_str());
+
+    if (!cp->error()) {
+
+        const FocusableList*                        windows;
+        FocusControl::Focusables::const_iterator    wit;
+        FocusControl::Focusables::const_iterator    wit_end;
+
+        for (screen = screens.begin(); screen != screens.end(); screen++) {
+
+            windows = FocusableList::getListFromOptions(**screen, opts|FocusableList::LIST_GROUPS);
+            wit = windows->clientList().begin();
+            wit_end = windows->clientList().end();
+
+            for ( ; wit != wit_end; wit++) {
+                if (typeid(**wit) == typeid(FluxboxWindow) && cp->match(**wit)) {
+                    matches.push_back(static_cast<const FluxboxWindow*>(*wit));
+                }
+            }
+        }
+
+        if (!matches.empty()) {
+            std::vector< const FluxboxWindow* >::const_iterator win;
+            for (win = matches.begin(); win != matches.end(); win++) {
+                result += "0x";
+                result += FbTk::StringUtil::number2HexString((*win)->clientWindow());
+                result += "\t";
+                result += (*win)->title().logical();
+                result += "\n";
+            }
+        } else {
+            result += "0\n";
+        }
+    } else {
+        result = "-1\t";
+        result += FbTk::StringUtil::number2String(cp->error_col());
+        result += "\n";
+    }
+
+
+    // write result to _FLUXBOX_ACTION_RESULT property
+    for (screen = screens.begin(); screen != screens.end(); screen++) {
+        (*screen)->rootWindow().changeProperty(atom_fbcmd_result, atom_utf8, 8,
+            PropModeReplace, (unsigned char*)result.c_str(), result.size());
+    }
+}
+
+
 } // end namespace FbCommands
